// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *
 *			   UNT Virtual Machine
 *		 A set of instructions that CPU and channel execute
 *
 ************************************************************************
 */

#ifndef _instruction_set_h
#define _instruction_set_h 1
#pragma interface

				// Word 1 of the CPU instruction
struct InstructionWord1
{
  unsigned short
    breg     : 4,			// 2nd register (source or index)
    areg     : 4,			// 1st register (respondent)
    opcode   : 7,			// Operation code
    reserved : 1;			// always zero
};

				// Word 2 of the CPU instruction
				// (if present)
struct InstructionWord2
{
  unsigned short
    ea	     : 10,			// Effective Address
    indirect : 1,			// Flag of indirect addressing
    svcop    : 5;			// Code for the SVC operation
};

enum OpCodes
{
				// One word instructions
  HALT,					// Halt the CPU
  DUMP,					// Dump regs and memory R(a) to R(b)
  LOADR,				// R(a) :=  R(b)
  ADD,					// R(a) +:= R(b)
  SUBT,					// R(a) -:= R(b)
  MULT,					// R(a) *:= R(b) (single word product)
  DIV,					// R(a) /:= R(b) (no trap on 0 divide)
  AND,					// R(a) &:= R(b)
  OR,					// R(a) |:= R(b)
  XOR,					// R(a) ^:= R(b)
  SHIFTL,				// R(a) <<:= R(b)
  SHIFTR,				// R(a) >>:= R(b)
  NOP,					// Naught
  IOPR,					// Illegal operation
				// Two word instructions
  TWO_WORD,				// 1st two-word code
  LOAD = TWO_WORD,			// R(a) := C(ea)
  LOADI,				// R(a) := ea
  STORE,				// C(ea) := R(a)
  BR,					// pc := ea
  BRNEG,				// if R(a) < 0 then pc := ea
  BRZERO,				// if R(a) = 0 then pc := ea
  BRPOS,				// if R(a) > 0 then pc := ea
  BRSUBR,				// R(a) := pc+2; pc := ea
  READ,					// Cause trap
  WRITE,				// Cause trap
  SVC,					// Cause SVC trap
  LAST_INSTR
};

/*
 *------------------------------------------------------------------------
 *			    Channel commands
 */

				// Note each device reads or writes records
				// of exactly page size. The I/O buffer should
				// be on the page boundary. That's why it's
				// enough to specify only page no of I/O buffer

				// Channel Command Word
struct CCW
{
  unsigned short
    page_no    : 10,			// Page no to be used in the op
    keep_going : 1,			// CCW chain flag
    opcode     : 4,			// Operation code
    reserved   : 1;			// always zero
};

enum CCWOpCodes
{
  ILLEGAL_CCW,				// Illegal operation
  SETMAP,				// Switch I/O address mapping
  SETPTBR,				// Set page base reg | for address
  SETPTL,				// Set page len	     | translation
  SEEKOP,				// Seek operation for disk,
  SKIPOP = SEEKOP,			// Skip, CR for printer
  READOP,				// Read
  WRITEOP,				// Write
  LAST_CCW_CODE
};
  
#endif

